@@ -2,8 +2,8 @@ import torch
 import numpy as np
 
 from msdm.core.distributions import DictDistribution
-from msdm.core.problemclasses.pomdp import TabularPOMDP
-from msdm.core.problemclasses.pomdp.finitestatecontroller import StochasticFiniteStateController
+from msdm.core.pomdp import TabularPOMDP
+from msdm.core.pomdp.finitestatecontroller import StochasticFiniteStateController
 from msdm.core.algorithmclasses import Learns, Result
 
 from msdm.algorithms.fscgradientascent import stochastic_fsc_policy_evaluation_exact
