@@ -1,12 +1,14 @@
+from functools import lru_cache
 import random
 import copy
 import warnings
 from collections import defaultdict
 from abc import ABC, abstractmethod
 from typing import Callable
+from msdm.core.mdp import TabularMarkovDecisionProcess, FunctionalPolicy, \
+        MarkovDecisionProcess, HashableState
 from msdm.core.utils.dictutils import defaultdict2
 from msdm.core.distributions import DictDistribution
-from msdm.core.problemclasses.mdp import MarkovDecisionProcess, TabularPolicy, HashableState
 from msdm.core.algorithmclasses import Plans, PlanningResult
 
 class LRTDP(Plans):
@@ -93,28 +95,51 @@ class LRTDP(Plans):
         else:
             self.res.event_listener = None
 
-    def _tear_down_plan_on(self, mdp, heuristic):
+    def _tear_down_plan_on(
+        self,
+        mdp : TabularMarkovDecisionProcess,
+        heuristic
+    ):
         res = self.res
 
-        # put together policy, q-values and initial value
-        res.policy = {}
-        res.Q = defaultdict(lambda : dict())
-        for s in res.V.keys():
-            res.policy[s] = DictDistribution.deterministic(self.policy(mdp, s))
+        q_values = defaultdict(lambda : dict())
+        policy_dict = {}
+        for s in self.res.V.keys():
+            policy_dict[s] = self.policy(mdp, s)
             for a in mdp.actions(s):
-                res.Q[s][a] = self.Q(mdp, s, a)
+                q_values[s][a] = self.Q(mdp, s, a)
+        res.Q = q_values
 
-        res.policy = DefaultTabularPolicy.with_default(
-            policy_dict=res.policy,
-            default_generator=lambda s: DictDistribution.uniform(mdp.actions(s))
-        )
+        @FunctionalPolicy
+        @lru_cache(maxsize=None)
+        def policy(s):
+            try:
+                action = policy_dict[s]
+                return DictDistribution.deterministic(action)
+            except KeyError:
+                pass
+            max_actions = []
+            max_val = float('-inf')
+            for a in mdp.actions(s):
+                ns_dist = mdp.next_state_dist(s, a)
+                val = ns_dist.expectation(
+                    lambda ns : mdp.reward(s, a, ns) + mdp.discount_rate*heuristic(ns)
+                ) 
+                if val > max_val:
+                    max_actions = [a]
+                elif val == max_val:
+                    max_actions.append(a)
+                max_val = max(val, max_val)
+            return DictDistribution.uniform(max_actions)
+
+        res.policy = policy
         res.initial_value = sum([res.V[s0]*p for s0, p in mdp.initial_state_dist().items()])
 
         #clear result
         self.res = None
         return res
 
-    def lrtdp(self, mdp, heuristic=None, iterations=None):
+    def lrtdp(self, mdp : MarkovDecisionProcess, heuristic=None, iterations=None):
         # Ghallab, Nau, Traverso: Algorithm 6.17
         self.res.V = defaultdict2(heuristic)
         self.res.action_orders = dict()
@@ -132,7 +157,7 @@ class LRTDP(Plans):
         else:
             self.res.converged = True
 
-    def lrtdp_trial(self, mdp, s):
+    def lrtdp_trial(self, mdp : MarkovDecisionProcess, s):
         # Ghallab, Nau, Traverso: Algorithm 6.17
         visited = [s, ]
         while not self.res.solved[s]:
@@ -141,7 +166,7 @@ class LRTDP(Plans):
             visited.append(s)
 
             # Terminal states are solved.
-            if mdp.is_terminal(s):
+            if mdp.is_absorbing(s):
                 self.res.solved[s] = True
             if len(visited) > self.max_trial_length:
                 break
@@ -187,13 +212,13 @@ class LRTDP(Plans):
         '''
         self.res.V[s] = max(self.Q(mdp, s, a) for a in mdp.actions(s))
 
-    def Q(self, mdp, s, a):
-        if mdp.is_terminal(s):
+    def Q(self, mdp: MarkovDecisionProcess, s, a):
+        if mdp.is_absorbing(s):
             return 0
         q = 0
         for ns, prob in mdp.next_state_dist(s, a).items():
             future = 0
-            if not mdp.is_terminal(ns):
+            if not mdp.is_absorbing(ns):
                 future = self.res.V[ns]
             q += prob * (mdp.reward(s, a, ns) + mdp.discount_rate*future)
         return q
@@ -210,21 +235,6 @@ class LRTDP(Plans):
             self.res.action_orders[s] = action_list
         return max(action_list, key=lambda a: self.Q(mdp, s, a))
 
-# TODO: this is a copy from laostar_refactor, we should consolidate
-# once this is finalized
-class DefaultTabularPolicy(TabularPolicy):
-    @classmethod
-    def with_default(cls, policy_dict, default_generator):
-        instance = DefaultTabularPolicy(policy_dict)
-        instance.default_generator = default_generator
-        return instance
-
-    def action_dist(self, s):
-        try:
-            return self[s]
-        except KeyError:
-            return self.default_generator(s)
-
 class LRTDPEventListener(ABC):
     @abstractmethod
     def end_of_lrtdp_trial(self, localvars):
