@@ -1,4 +1,5 @@
-from msdm.core.distributions.distributions import Distribution, FiniteDistribution
+from msdm.core.distributions.distributions import Distribution, \
+    FiniteDistribution, ImplicitDistribution
 from msdm.core.distributions.discretefactortable import \
     DiscreteFactorTable
 from msdm.core.distributions.dictdistribution import DictDistribution, DeterministicDistribution, UniformDistribution
